﻿using System;
using System.IO;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace XNACS1Lib {
    /// <summary>
    /// Inherits from XNACS1PrimitiveSet, allows a user to define behavior for a group of particles.
    /// </summary>
    public abstract partial class XNACS1ParticleEmitter : XNACS1Primitive {
        /// <summary>
        /// Emits particles at the center of the emitter at a constant rate, which fade out according to InitialParticleLife.
        /// If the emitter is moving around this will look like a tail.  Works best if attached to other primitives.
        /// </summary>
        public class TailEmitter : XNACS1ParticleEmitter {
            /// <summary>
            /// Constructor for TailEmitter
            /// </summary>
            /// <param name="center"></param>
            /// <param name="initialParticleLife"></param>
            /// <param name="particleSize"></param>
            /// <param name="tex"></param>
            /// <param name="defaultColor"></param>
            public TailEmitter(Vector2 center, int initialParticleLife, float particleSize, string tex, Color defaultColor)
                : base(center, initialParticleLife, particleSize, tex, defaultColor) {
                m_AutoEmit = false;
            }



            Vector2 m_LastCenter;

            /// <summary>
            /// Overloaded update functionality to make it only emit if the emitter has moved, so that the tail wont bunch up if it stops moving.
            /// </summary>
            protected override void Update() {
                
                base.Update();
                if (m_LastCenter != this.Center)
                    Emit();
                m_LastCenter = this.Center;
            }


            /// <summary>
            /// Overloaded UpdateParticle to cause particles to fade as they die.
            /// </summary>
            /// <param name="particle"></param>
            protected override void UpdateParticle(XNACS1Particle particle) {
                int temp = (255 * particle.Life) / m_DefaultLife;
                if (temp > 255) temp = 255;
                else if (temp < 0) temp = 0;
                particle.m_TextureColor.A = (byte)temp;
            }
        }

        /// <summary>
        /// Emits explosions when Explode() is called.
        /// </summary>
        public class ExplodeEmitter : XNACS1ParticleEmitter {
            float m_Radius;
            float m_ExplodeSpeed;

            /// <summary>
            /// Constructor for explode emitter.
            /// Explode emitters do nothing until the Explode() method is called.
            /// </summary>
            /// <param name="center"></param>
            /// <param name="initialParticleLife"></param>
            /// <param name="particleSize"></param>
            /// <param name="tex"></param>
            /// <param name="defaultColor"></param>
            /// <param name="radius">Approximate Radius of explosion</param>
            public ExplodeEmitter(Vector2 center, int initialParticleLife, float particleSize, string tex, Color defaultColor, float radius)
                : base(center, initialParticleLife, particleSize, tex, defaultColor) {
                AutoEmitEnabled = false;
                m_Radius = radius;
                m_ExplodeSpeed = radius / initialParticleLife;
            }


            /// <summary>
            /// Get or set the speed at which particles explode outward.
            /// Default is radius / life;
            /// </summary>
            public float ExplodeSpeed {
                get { return m_ExplodeSpeed; }
                set { m_ExplodeSpeed = value; }
            }

            /// <summary>
            /// Creates particles which explode outward from the center of the emitter at varying velocities.
            /// </summary>
            /// <param name="NumParticles">Number of particles to create.</param>
            public void Explode(int NumParticles) {
                for (int i = 0; i < NumParticles; i++) {
                    Emit();
                }
            }

            /// <summary>
            /// Overloaded NewParticle Method, gives explosion particles initial velocity and sets
            /// ShouldTravel = true.
            /// </summary>
            /// <param name="newParticle"></param>
            protected override void NewParticle(XNACS1Particle newParticle) {
                newParticle.CenterX += XNACS1Base.RandomFloat(-m_Radius / 10, m_Radius / 10);
                newParticle.CenterY += XNACS1Base.RandomFloat(-m_Radius / 10, m_Radius / 10);
                Vector2 randomVelocity = new Vector2(XNACS1Base.RandomFloat(-1, 1), XNACS1Base.RandomFloat(-1, 1));
                randomVelocity *= m_ExplodeSpeed;
                newParticle.Velocity = randomVelocity;
                newParticle.ShouldTravel = true;

            }

            /// <summary>
            /// Overloaded UpdateParticle method, makes particles fade out according to what
            /// percentage of the default life they have remaining.
            /// </summary>
            /// <param name="particle"></param>
            protected override void UpdateParticle(XNACS1Particle particle) {
                int temp = (255 * particle.Life) / m_DefaultLife;
                if (temp > 255) temp = 255;
                else if (temp < 0) temp = 0;
                particle.m_TextureColor.A = (byte)temp;
            }
        }
    }

}