using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace XNACS1Lib
{
    /// <summary>
    /// Enum data type for collision results between a Primitive and the world bound 
    /// Status returned by ClampAtWorldBound() and CollideWorldBound.
    /// </summary>
    public enum BoundCollideStatus
    {
        /// <summary>
        /// Primitive has collided with the left-world boundary.
        /// </summary>
        CollideLeft,

        /// <summary>
        /// Primitive has collided with the right-world boundary.
        /// </summary>
        CollideRight,

        /// <summary>
        /// Primitive has collided with the bottom-world boundary.
        /// </summary>
        CollideBottom,
        
        /// <summary>
        /// Primitive has collided with the top-world boundary.
        /// </summary>
        CollideTop,
        
        /// <summary>
        /// Primitive is inside the world boundary.
        /// </summary>
        InsideBound
    };

    /// <summary>
    /// Define SpriteSheet Animation
    /// </summary>
    public enum SpriteSheetAnimationMode {
        /// <summary>
        /// Animate forward in frame numbers (loops from begin to end)
        /// </summary>
        AnimateForward,

        /// <summary>
        /// Animate backward in frame numbers (loops from end to begin)
        /// </summary>
        AnimateBackward,

        /// <summary>
        /// Swing the animation in frame numbers (goes back and forth between begin and end)
        /// </summary>
        AnimateSwing
    };

    /// <summary>
    /// This is the base class for your program. To work with the XNACS1Lib, your main class must subclass
    /// from this class.
    /// </summary>
    /// <remark>
    /// This is the base class for your program. To work with the XNACS1Lib, your main class must subclass
    /// from this class.
    /// </remark>
    public class XNACS1Base : IDisposable
    {
        #region Provided a public class to gain access to the "Global World" functions
        /// <summary>
        /// This this the class for setting, inquiring, and changing information corresponding
        /// to the "World", including:
        /// Coordinate System, Background color/texture, Clamping/colliding with the boundary, and removing
        /// all primitives from the AutoDraw set.
        /// </summary>
        public static class World
        {   
            /// <summary>
            /// Number of ticks in one second.
            /// </summary>
            public const int TicksInASecond = 40;

            private static bool mPaused = false;
    
            /// <summary>
            /// Sets the background color.
            /// </summary>
            /// <param name="bgColor">The specified color for the entire background.</param>
            public static void SetBackgroundColor(Color bgColor)
            {
                sm_GraphicsAccess.m_WorldBound.Color = bgColor;
            }


            /// <summary>
            /// Defines the world coordinate system to draw in.
            /// </summary>
            /// <param name="min">Coordinate position of the lower left corner of the drawing window.</param>
            /// <param name="width">Width represented by the drawing window.</param>
            static public void SetWorldCoordinate(Vector2 min, float width)
            {
                sm_GraphicsAccess.RequestedWorldMin = min;
                sm_GraphicsAccess.RequestedWorldWidth = width;
                sm_GraphicsAccess.CreateDrawHelper();
            }

            /// <summary>
            /// Set the background texture. The texture image will cover the entire background such that the 
            /// background color will not be visible.
            /// </summary>
            /// <param name="tex">name of the background texture image.</param>
            public static void SetBackgroundTexture(String tex)
            {
                sm_GraphicsAccess.m_WorldBound.Texture = tex;
            }

            /// <summary>
            /// Pausing the World Update (ShouldTravel primitive will stop moving).
            /// </summary>
            public static bool Paused
            {
                set { mPaused = value; }
                get { return mPaused; }
            }

            /// <summary>
            /// Get the coodinate position that corresponds to the lower-left corner of the world.
            /// </summary>  
            /// <seealso cref="WorldMax"/>
            /// <seealso cref="WorldDimension"/>
            public static Vector2 WorldMin
            {
                get { return sm_GraphicsAccess.m_WorldBound.MinBound; }
            }

            /// <summary>
            /// Get the coordinate position that corresponds to the upper-right corner of the world.
            /// </summary>
            /// <seealso cref="WorldMin"/>
            /// <seealso cref="WorldDimension"/>
            public static Vector2 WorldMax
            {
                get { return sm_GraphicsAccess.m_WorldBound.MaxBound; }
            }

            /// <summary>
            /// Get the dimension (width/height) of the world. 
            /// </summary>
            /// <seealso cref="WorldMin"/>
            /// <seealso cref="WorldMax"/>
            public static Vector2 WorldDimension
            {
                get { return sm_GraphicsAccess.m_WorldBound.m_Size; }
            }

            /// <summary>
            /// Returns true if the inputPrimitive is outside of the worldBound. 
            /// </summary>
            /// <param name="inputPrimitive">the primitive to check</param>
            /// <returns>True if inputPrimitive is outside the world bound, otherwise returns false.</returns>
            public static bool OutsideWorldBound(XNACS1Primitive inputPrimitive)
            {
                Vector2 min = sm_GraphicsAccess.m_WorldBound.MinBound;
                Vector2 max = sm_GraphicsAccess.m_WorldBound.MaxBound;

                return (inputPrimitive.MinBound.X > max.X) ||
                       (inputPrimitive.MaxBound.X < min.X) ||
                       (inputPrimitive.MinBound.Y > max.Y) ||
                       (inputPrimitive.MaxBound.Y < min.Y);
            }

            /// <summary>
            /// Collide the inputPrimitive with the worldBound and return which of the worldBound (left/right/top/bottom or inside)
            /// did this primitive intersect with.
            /// </summary>
            /// <param name="inputPrimitive">Primitive to be tested</param>
            /// <returns>The side upon which the primitive has collided with (or if the primitive is inside the world bound).</returns>
            public static BoundCollideStatus CollideWorldBound(XNACS1Primitive inputPrimitive)
            {
                Vector2 min = sm_GraphicsAccess.m_WorldBound.MinBound;
                Vector2 max = sm_GraphicsAccess.m_WorldBound.MaxBound;

                if (inputPrimitive.MinBound.X < min.X)
                    return BoundCollideStatus.CollideLeft;
                if (inputPrimitive.MaxBound.X > max.X)
                    return BoundCollideStatus.CollideRight;
                if (inputPrimitive.MinBound.Y < min.Y)
                    return BoundCollideStatus.CollideBottom;
                if (inputPrimitive.MaxBound.Y > max.Y)
                    return BoundCollideStatus.CollideTop;

                return BoundCollideStatus.InsideBound;
            }

            /// <summary>
            /// Ensures that the inputPrimitive is kept within bounds of the world. If inputPrimitive position is outside
            /// of the world bound, This function will determine the distances to the closes bound and force the inputPrimitive to
            /// be insde that world bound.
            /// </summary>
            /// <param name="inputPrimitive">The primtive to be tested</param>
            /// <returns>Which of the world bound that the inputPrimitive has moved out of.</returns>
            public static BoundCollideStatus ClampAtWorldBound(XNACS1Primitive inputPrimitive)
            {
                BoundCollideStatus status = CollideWorldBound(inputPrimitive);
                if (status != BoundCollideStatus.InsideBound)
                {
                    Vector2 size = (inputPrimitive.MaxBound - inputPrimitive.MinBound) * 0.5f;
                    Vector2 max = new Vector2(WorldMax.X - size.X, WorldMax.Y - size.Y);
                    Vector2 min = new Vector2(WorldMin.X + size.X, WorldMin.Y + size.Y);
                    Vector2 pos = new Vector2(inputPrimitive.Center.X, inputPrimitive.Center.Y);
                    if (pos.X > max.X) pos.X = max.X;
                    if (pos.Y > max.Y) pos.Y = max.Y;
                    if (pos.X < min.X) pos.X = min.X;
                    if (pos.Y < min.Y) pos.Y = min.Y;
                    inputPrimitive.Center = pos;
                }
                return status;
            }

            /// <summary>
            /// Remove all Primitives from the default draw set. After this funciton, you will observe a blank screen.
            /// </summary>
            public static void RemoveAllFromDrawSet()
            {
                sm_GraphicsAccess.m_DrawHelper.RemoveAllFromSet();
            }

            /// <summary>
            /// When on the Zune, run in landscape mode (default is portait mode)
            /// </summary>
            /// <param name="toLandscape">if true, sets the Zune to run in landscape mode</param>
            public static void SetZuneToLandscape(bool toLandscape)
            {
                sm_GraphicsAccess.m_ZuneInLandscape = toLandscape;
                sm_GraphicsAccess.CreateDrawHelper();
            }

        };

        #endregion
        #region Private implementation subclasses have no need to have knowledge or to access any of these

        static internal Random m_Ran = new Random();
        static internal XNACS1Internal sm_GraphicsAccess = null;

        #endregion  private data of XNACS1Lib class


        /// <summary>
        /// This is the constructor of the base class for the library.
        /// </summary>
        public XNACS1Base()
        {
            XNACS1Base.sm_GraphicsAccess = new XNACS1Internal(this);
        }

        internal void InternalUpdate() { UpdateWorld(); }
        internal void InternalInit() { InitializeWorld(); }
        internal void InternalDraw() { DrawWorld(); }

        /// <summary>
        /// Immediately release all resources. This method is called automatically during the destruction of the application.
        /// </summary>
        public void Dispose() { }
        

        #region Methods subclass should override and should call base.methods

        /// <summary>
        ///  Subclass should override this to define the World Cooridnate origin and width, and to allocate
        /// and otherwise initialize the application state. This method is called once at the beginning of the application.
        /// </summary>
        protected virtual void InitializeWorld() { return; }

        /// <summary>
        ///  Subclass should override this method to update the world. This function is called 40 times a second.
        /// </summary>
        protected virtual void UpdateWorld() { return; }

        /// <summary>
        /// 
        /// subclass can override this to customize the redrawing of the world. This function is called 40 times a second.
        /// </summary>
        protected virtual void DrawWorld() { return; }

        #endregion


        #region methods to support running of the app.
        /// <summary>
        /// Begins the application. As programmer, you should instantiate an instance of XNACS1Base object, and then call this method to
        /// begin the application. This is the first method you should call to begin the application.
        /// </summary>
        public void Run()
        {
            sm_GraphicsAccess.Run();
        }

        /// <summary>
        /// Quits the application. This is the last method you will call.
        /// </summary>
        public void Exit()
        {
            sm_GraphicsAccess.Exit();
        }
        #endregion


        #region public utility methods

        /// <summary>
        /// For accessing the state of the XBOX 360 GamePad Controller. In the absence of a GamePad controller, the keyboard
        /// will be polled according to the default mapping.
        /// </summary>
        static public XNACS1Lib.GamePadSupport.GamePadStruct GamePad { get { return XNACS1LibInputManager.GamePad; } }

        /// <summary>
        /// Sets the top status echo font color.
        /// </summary>
        /// <param name="topEchoColor">Color of the top echo font.</param>
        static public void SetTopEchoColor(Color topEchoColor)
        {
            sm_GraphicsAccess.m_FontManager.TopEchoFontColor = topEchoColor;
        }
        
        /// <summary>
        /// Echo message to the status area located at the top of the drawing window.
        /// </summary>
        /// <param name="msg">Message to be echoed.</param>
        static public void EchoToTopStatus(String msg)
        {
            sm_GraphicsAccess.m_TopStatus = msg;
        }


        /// <summary>
        /// Sets the bottom status echo font color.
        /// </summary>
        /// <param name="bottomEchoColor">Color of the bottom echo font.</param>
        static public void SetBottomEchoColor(Color bottomEchoColor)
        {
            sm_GraphicsAccess.m_FontManager.BottomEchoFontColor = bottomEchoColor;
        }
        
        /// <summary>
        /// Echo message to the status area located at the bottom of the drawing window.
        /// </summary>
        /// <param name="msg">Message to be echoed.</param>
        static public void EchoToBottomStatus(string msg)
        {
            //msg = "Size=" + sm_GraphicsAccess.m_DrawHelper.DrawSetSize() + " || " + msg;
            sm_GraphicsAccess.m_BottomStatus = msg;
        }

        /// <summary>
        /// Play the spcified audio cue for the specified length of seconds.
        /// </summary>
        /// <param name="cueName">Name of the audio cue (wav filename without the .wav extension).</param>
        public static void PlayACue(String cueName)
        {
               sm_GraphicsAccess.mAudioManager.PlaySound(cueName);
        }

        /// <summary>
        /// Plays the specific audo file name continuously as the background audio.
        /// </summary>
        /// <param name="bgAudio">Name of the file to be used as background music.</param>
        /// <param name="level">The loudness of bg music (1.0 is loudest, 0.0 is silent).</param>
        /// <remarks>Pass in a null string (null) to stop the background audio.</remarks>
        public static void PlayBackgroundAudio(String bgAudio, float level)
        {
            sm_GraphicsAccess.mAudioManager.PlayBackgroundAudio(bgAudio, level);
        }

#if _NOTANYMORE_
        #region Drawing Support

        /// <summary>
        /// Draws a circle.
        /// </summary>
        /// <param name="center">Center of the circle.</param>
        /// <param name="radius">Radius of the circle.</param>
        /// <param name="rotation">Rotation of the circle (about the center, in degree).</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColoe">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the circle.</param>
        public static void DrawCircle(Vector2 at, float radius, float rot, Color outsideColor, Color centerColor, String texName)
        {
            sm_GraphicsAccess.m_DrawHelper.DrawCircle(at, radius, rot, outsideColor, centerColor, texName);
        }

        /// <summary>
        /// Draws a general non-axis aligned rectangle.
        /// </summary>
        /// <param name="pa">One end of rectangle (center of edge 1).</param>
        /// <param name="pb">Opposite end of rectangle (center of edge 2).</param>
        /// <param name="edgeWidth">Width of the edges where pa and pb are located (length of the rectangle is defined by the distance between pa and pb).</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColoe">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the rectangle.</param>
        public static void DrawRectangle(Vector2 pa, Vector2 pb, float edgeWidth, Color outsideColor, Color centerColor, String texName)
        {
            sm_GraphicsAccess.m_DrawHelper.DrawRectangle(pa, pb, edgeWidth, outsideColor, centerColor, texName);
        }

        /// <summary>
        /// Draws a roated rectangle.
        /// </summary>
        /// <param name="center">Center of the rectangle.</param>
        /// <param name="xWidth">Horizontal width of the rectangle.</param>
        /// <param name="yHeight">Vertical height of the rectangle.</param>
        /// <param name="rot">Rotation in degrees.</param>
        /// <param name="edgeWidth">Width of the edges where pa and pb are located (length of the rectangle is defined by the distance between pa and pb).</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColoe">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the rectangle.</param>
        public static void DrawRectangle(Vector2 center, float xWidth, float yHeight, float rot, Color outsideColor, Color centerColor, String texName)
        {
            sm_GraphicsAccess.m_DrawHelper.DrawRectangle(center, xWidth, yHeight, rot, outsideColor, centerColor, texName);
        }
#endregion for drawing support
#endif

        #endregion

        #region Random number support
        /// <summary>
        /// Returns a random number between 0.0 to 1.0
        /// </summary>
        /// <returns>A random number between 0.0 to 1.0.</returns>
        public static float RandomFloat()
        {
            return (float)(m_Ran.NextDouble());
        }
        /// <summary>Returns a random float between 0 and max.  This function may return 0, or may return
        /// a number that's close to max, but will never return max itself
        /// </summary>
        /// <param name="max">Max number to be returned.</param>
        /// <returns>A random number between 0 and max.</returns>
        public static float RandomFloat(float max)
        {
            return (max * RandomFloat());
        }
        /// <summary>
        /// Returns a float between min and max.  This function may return min, or may return
        /// a number that's close to max, but will never return max itself
        /// </summary>
        /// <param name="min">Min number to be returned.</param>
        /// <param name="max">Max number to be returned.</param>
        /// <returns>A random number between min and max.</returns>
        public static float RandomFloat(float min, float max)
        {
            float s = max - min;
            return min + (s * RandomFloat());
        }
        /// <summary>
        /// Returns a random integer between 0 and some VERY LARGE number.
        /// </summary>
        /// <returns>A random integer between 0 and some very large number.</returns>
        public static int RandomInt()
        {
            return m_Ran.Next();
        }
        /// <summary>
        /// Returns a random integer between 0 and max.
        /// </summary>
        /// <param name="max">Max integer to be returned.</param>
        /// <returns>Returns a random integer between 0 and max.</returns>
        public static int RandomInt(int max)
        {
            return m_Ran.Next(max);
        }
        /// <summary>
        /// Returns a random integer between min and max.
        /// </summary>
        /// <param name="min">Min integer to be returned.</param>
        /// <param name="max">Max integer to be returned.</param>
        /// <returns>Returns a random integer between min and max.</returns>
        public static int RandomInt(int min, int max)
        {
            return m_Ran.Next(min, max);
        }
        #endregion
    }
}